package com.loit.common.script;

import com.loit.common.script.dto.BackendFrontEnum;
import com.loit.common.script.dto.DeployInfoDataDTO;
import com.loit.common.script.dto.FrontGroupEnum;
import com.loit.common.script.dto.FrontInstanceDto;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class GeneratorScript {


    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";


    protected static String root_path_full = root_path + "\\server";

    private static Map<String, FrontInstanceDto> frontInstanceDtoMap = new LinkedHashMap<>();


    public static void main(String[] args) {

        try {

            //String filePathStr = Thread.currentThread().getContextClassLoader().getResource("deployInfo.xlsx").getPath();
            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\deployInfo.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 1, 0);
            List<DeployInfoDataDTO> grayVersionDataDTOList = ei.getDataList(DeployInfoDataDTO.class);

            if (ListUtil.isEmpty(grayVersionDataDTOList)) {
                return;
            }


            for (DeployInfoDataDTO grayVersionDataDTO : grayVersionDataDTOList) {
                String needCreateInstanceSql = grayVersionDataDTO.getNeedCreateInstanceSql();
                if ("no".equals(needCreateInstanceSql)) {
                    continue;
                }
                buildInitEnv(grayVersionDataDTO);
                buildBackendConfig(grayVersionDataDTO);
                buildFrontConfig(grayVersionDataDTO);
            }


            //对前端实例进行分组
            for (DeployInfoDataDTO deployDataDTO : grayVersionDataDTOList) {

                String needCreateInstanceSql = deployDataDTO.getNeedCreateInstanceSql();
                if ("no".equals(needCreateInstanceSql)) {
                    continue;
                }

                BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(deployDataDTO.getBackendFrontType());


                String serviceName = deployDataDTO.getServiceName();
                FrontInstanceDto frontInstanceDto = frontInstanceDtoMap.get(serviceName);
                if (frontInstanceDto == null) {
                    frontInstanceDto = new FrontInstanceDto();
                    String serviceNameSub = "loit-" + serviceName.replace("-web", "").replace("-h5", "");
                    String serviceNameCookie = "loit" + serviceName.replaceAll("-", "");
                    frontInstanceDto.setServiceName(serviceName);
                    frontInstanceDto.setServiceNameSub(serviceNameSub);
                    frontInstanceDto.setServiceNameCookie(serviceNameCookie);
                    frontInstanceDto.setServiceNameAbb(deployDataDTO.getServiceNameAbb());
                    frontInstanceDto.setBackendFrontType(backendFrontType);
                    frontInstanceDtoMap.put(serviceName, frontInstanceDto);
                }

                FrontGroupEnum frontGroup = FrontGroupEnum.getEnumByCode(deployDataDTO.getFrontGroup());
                String ipAndPort = deployDataDTO.getFixedIp() + ":" + deployDataDTO.getPort();
                if (FrontGroupEnum.gray.equals(frontGroup)) {
                    frontInstanceDto.getGrayList().add(ipAndPort);
                }

                if (FrontGroupEnum.normal.equals(frontGroup)) {
                    frontInstanceDto.getNormalList().add(ipAndPort);
                }
            }


            //生成前端nginx 配置文件
            frontInstanceDtoMap.forEach((key, frontInstanceDto) -> {
                if (StringUtils.isBlank(key)) {
                    return;
                }

                Map model = new HashMap();
                model.put("serviceNameSub", frontInstanceDto.getServiceNameSub());
                model.put("serviceNameCookie", frontInstanceDto.getServiceNameCookie());
                model.put("serviceName", frontInstanceDto.getServiceName());
                model.put("serviceNameAbb", frontInstanceDto.getServiceNameAbb());
                model.put("grayList", frontInstanceDto.getGrayList());
                model.put("normalList", frontInstanceDto.getNormalList());
                BackendFrontEnum backendFrontType = frontInstanceDto.getBackendFrontType();
                model.put("backendFrontType", backendFrontType.getCode());

                if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
                    // 构建gateway 配置
                    buildGatewayConfig(model);
                } else if (BackendFrontEnum.web.equals(backendFrontType)) {
                    //创建政务网nginx配置
                    buildFrontZwwNginxConfig(model);
                    buildFrontHlwNginxConfig(model);
                } else if (BackendFrontEnum.h5.equals(backendFrontType)) {
                    //创建政务网nginx配置
                    buildFrontZwwNginxConfig(model);
                }
            });

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 生成InitEnv
     *
     * @throws IOException
     */
    private static void buildInitEnv(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String fileDir = root_path_full + "\\" + grayVersionDataDTO.getFixedIp();
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String nginxConfigDir = root_path_full + "\\" + grayVersionDataDTO.getFixedIp() + "\\nginxConfig";
        File nginxConfigFolder = new File(nginxConfigDir);
        if (!nginxConfigFolder.exists()) {
            nginxConfigFolder.mkdirs();
        }

        String filePath = fileDir + "\\initEnv.sh";
        File file = new File(filePath);
        if (!file.exists()) {
            FileUtils.write(filePath, "#!/bin/bash ");
        }

        String deployPath = grayVersionDataDTO.getDeployPath();
        if (StringUtils.isNoneBlank(deployPath)) {
            FileUtils.appendNewLine(filePath, "mkdir -p " + deployPath);
            FileUtils.appendNewLine(filePath, "rm -rf " + deployPath + "?");
            BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
            if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
                FileUtils.appendNewLine(filePath, "mkdir -p " + deployPath + "/logs");
                FileUtils.appendNewLine(filePath, "rm -rf " + deployPath + "/logs?");
            }
        }


        String nginxConfigShFileName = fileDir + "\\copyNginxConfig.sh";
        String nginxConfigShResult = FreeMarkerUtils.process("sh\\copyNginxConfig.ftl", null);
        FileUtils.write(nginxConfigShFileName, nginxConfigShResult);

    }


    /**
     * 生成bootstrap.properties 及其 sh
     */
    private static void buildBackendConfig(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String serviceName = grayVersionDataDTO.getServiceName();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String port = grayVersionDataDTO.getPort();
        String deployPath = grayVersionDataDTO.getDeployPath();
        String deployJar = grayVersionDataDTO.getDeployJar();
        String serviceNameAbb = grayVersionDataDTO.getServiceNameAbb();
        String springProfilesActive = grayVersionDataDTO.getSpringProfilesActive();

        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
        if (!BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String folderName = deployPath.replace("/home/soft/", "");
        String fileDir = root_path_full + "\\" + fixedIp + "\\" + folderName;
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String bootstrapFileName = fileDir + "\\bootstrap.properties";

        Map model = new HashMap();
        model.put("port", port);
        model.put("serviceName", serviceName);
        model.put("deployPath", deployPath);
        model.put("deployJar", deployJar);
        model.put("serviceNameAbb", serviceNameAbb);
        model.put("springProfilesActive", springProfilesActive);

        String result = FreeMarkerUtils.process("sh\\bootstrap.ftl", model);
        FileUtils.write(bootstrapFileName, result);


        String deployShFileName = fileDir + "\\deploy-" + serviceName + "-" + port + ".sh";
        String deployShResult = FreeMarkerUtils.process("sh\\deploy-sh.ftl", model);
        FileUtils.write(deployShFileName, deployShResult);
    }


    /**
     * 生成bootstrap.properties 及其 sh
     */
    private static void buildFrontConfig(DeployInfoDataDTO grayVersionDataDTO) throws IOException {

        String serviceName = grayVersionDataDTO.getServiceName();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String port = grayVersionDataDTO.getPort();
        String deployPath = grayVersionDataDTO.getDeployPath();
        String deployJar = grayVersionDataDTO.getDeployJar();
        String serviceNameAbb = grayVersionDataDTO.getServiceNameAbb();
        String nginxConfigName = grayVersionDataDTO.getNginxConfigName();
        String frontGroup = grayVersionDataDTO.getFrontGroup();


        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
        if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            return;
        }

        String folderName = deployPath.replace("/home/soft/", "");
        String fileDir = root_path_full + "\\" + fixedIp + "\\" + folderName;
        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        Map model = new HashMap();
        model.put("port", port);
        model.put("serviceName", serviceName);
        model.put("deployPath", deployPath);
        model.put("deployJar", deployJar);
        model.put("serviceNameAbb", serviceNameAbb);
        model.put("nginxConfigName", nginxConfigName);
        model.put("frontGroup", frontGroup);

        String deployShFileName = fileDir + "\\deploy-" + serviceName + "-" + port + ".sh";
        String deployShResult = FreeMarkerUtils.process("sh\\deploy-web-sh.ftl", model);
        if ("portal-web".equals(serviceName)) {
            deployShResult = FreeMarkerUtils.process("sh\\deploy-portal-web-sh.ftl", model);
        }

        FileUtils.write(deployShFileName, deployShResult);

        if (StringUtils.isNotBlank(nginxConfigName)) {
            String nginxName = root_path_full + "\\" + fixedIp + "\\nginxConfig" + "\\" + nginxConfigName;
            String nginxConfigResult = FreeMarkerUtils.process("sh\\nginx_config.ftl", model);
            FileUtils.write(nginxName, nginxConfigResult);
        }


    }


    private static void buildFrontZwwNginxConfig(Map model) {
        String nginxConfigDir = root_path_full + "\\zww-nginx.conf";
        FileUtils.appendNewLine(nginxConfigDir, "----------------------------------" + model.get("serviceName") + "----------------------------------");

        String nginxConfig = FreeMarkerUtils.process("sh\\zww-nginx-upstream.ftl", model);
        FileUtils.appendNewLine(nginxConfigDir, nginxConfig);
    }


    private static void buildFrontHlwNginxConfig(Map model) {

        String nginxConfigDir = root_path_full + "\\hlw-nginx.conf";
        FileUtils.appendNewLine(nginxConfigDir, "----------------------------------" + model.get("serviceName") + "----------------------------------");
        String nginxConfig = FreeMarkerUtils.process("sh\\hlw-nginx-upstream.ftl", model);
        FileUtils.appendNewLine(nginxConfigDir, nginxConfig);
    }


    private static void buildGatewayConfig(Map model) {

        String nginxConfigDir = root_path_full + "\\gateway.conf";
        FileUtils.appendNewLine(nginxConfigDir, "----------------------------------" + model.get("serviceName") + "----------------------------------");
        String nginxConfig = FreeMarkerUtils.process("sh\\gateway.ftl", model);
        FileUtils.appendNewLine(nginxConfigDir, nginxConfig);
    }
}
